#include <string>

#include "sudo_plugin.hxx"

#if SUDO_API_VERSION_MAJOR != 1
#error "Incompatible sudo plugin API major version: " SUDO_API_VERSION_MAJOR
#endif

#if SUDO_API_VERSION_MINOR < 15
#error "Incompatible sudo plugin API version: 1." SUDO_API_VERSION_MINOR ", need at least 1.15"
#endif

static sudo_conv_t converse;
static sudo_printf_t sudo_printf;

static int sudo_approve_command_open(unsigned int version, sudo_conv_t conversation,
			      sudo_printf_t sudo_plugin_printf,
			      char *const settings[], char *const user_info[],
			      int submit_optind, char *const submit_argv[],
			      char *const submit_envp[],
			      char *const plugin_options[], const char **errstr)
{
	converse = conversation;
	sudo_printf = sudo_plugin_printf;

	return 1;
}

// static void sudo_approve_command_close(void);

static int sudo_approve_command_check(char *const command_info[],
			       char *const run_argv[], char *const run_envp[],
			       const char **errstr)
{
	struct sudo_conv_message msg = {
		.msg_type = SUDO_CONV_PROMPT_ECHO_ON,
		.timeout = 0,
		.msg = nullptr,
	};
	struct sudo_conv_reply reply = { nullptr };

	std::string buf = "Command info:\n";

	for (auto i = command_info; *i != nullptr; i++) {
		buf.append(*i);
		buf.push_back('\n');
	}

	buf.append("Command to run:\n");
	for (auto i = run_argv; *i != nullptr; i++) {
		buf.append(*i);
		buf.push_back('\n');
	}

	buf.append("Run this command? [y/N]: ");

	msg.msg = buf.c_str();

	int ret = converse(1, &msg, &reply, nullptr);
	if (ret != 0) {
		return -1;
	}

	if (!reply.reply) {
		return -1;
	}

	if (*reply.reply == 'y' || *reply.reply == 'Y') {
		ret = 1;
	} else {
		ret = 0;
	}

	free(reply.reply);
	return ret;
}

static int sudo_approve_command_show_version(int verbose)
{
	return 1;
}

struct approval_plugin approve_command = {
	.type = SUDO_APPROVAL_PLUGIN,
	.version = SUDO_API_VERSION,
	.open = sudo_approve_command_open,
	.close = nullptr,
	.check = sudo_approve_command_check,
	.show_version = sudo_approve_command_show_version,
};
